#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "person.h"
#include "personio.h"
#include "personsort.h"


typedef enum
{
    SORT_KEY_ID,     /* 0 */
    SORT_KEY_NAME,   /* 1 */
    SORT_KEY_WEIGHT  /* 2 */
} sort_key_t;


static int usage(char *progname)
{
    fprintf(stderr,
        "Sort persons database via key (id, name or weight)\n"
        "\n"
        "Usage:\n"
        "  %s [-sortid | -sortnm | -sortw] [db-input] [report-output]\n"
        "\n"
        "Where default key is 'id'\n", progname);
    return 1;
} /* usage */


static int get_parameters(int argc, char **argv,
    sort_key_t *p_sort_key,
    char **p_fn_db,
    char **p_fn_rep
)
{
    int         rc = 0;
    sort_key_t  sort_key = SORT_KEY_ID;

    if (argc < 3)
    {
         rc = usage(argv[0]);
    }
    else
    {
         int   ai = 1;
         char  *opt = argv[ai];

         if (strcmp(opt, "-sortid") == 0)
         {
             sort_key = SORT_KEY_ID;      ai++;
         }
         else if (strcmp(opt, "-sortnm") == 0)
         {
             sort_key = SORT_KEY_NAME;    ai++;
         }
         else if (strcmp(opt, "-sortw") == 0)
         {
             sort_key = SORT_KEY_WEIGHT;  ai++;
         }

         if (ai + 2 != argc)
         {
             rc = usage(argv[0]);
         }
         else
         {
             *p_fn_db = argv[ai];
             *p_fn_rep = argv[ai + 1];
         }
    }
    *p_sort_key = sort_key;
    return rc;
} /* get_parameters */


static int person_cmp_id(person_t* p0, person_t *p1)
{
    return p0->numeric_id - p1->numeric_id;
}


static int person_cmp_name(person_t* p0, person_t *p1)
{
    return strcmp(p0->name, p1->name);
}


static int person_cmp_weight(person_t* p0, person_t *p1)
{
    int ret = 0;

    if (p0->weight < p1->weight)
    {
        ret = -1;
    } 
    else if (p0->weight > p1->weight)
    {
        ret = 1;
    }
    return ret;
}


static void free_persons(person_t *persons, int n_persons)
{
    int  pi;

    for (pi = 0;  pi < n_persons;  pi++)
    {
        person_t  *p = &persons[pi];
        free(p->name);
    }
    free(persons);
}


int main(int argc, char **argv)
{
    int         rc = 0;
    char        *fn_db;
    char        *fn_rep;
    person_t    *persons = NULL;
    int         n_persons = 0;
    sort_key_t  sort_key = SORT_KEY_ID;

    rc = get_parameters(argc, argv, &sort_key, &fn_db, &fn_rep);
    if (rc == 0)
    {
        n_persons = read_count_persons(fn_db, &persons);
        if (n_persons < 0)
        {
            rc = 1;
        }
        else
        {
            switch (sort_key)
            {
             case SORT_KEY_ID:
                sort_persons(persons, n_persons, &person_cmp_id);
                break;
             case SORT_KEY_NAME:
                sort_persons(persons, n_persons, &person_cmp_name);
                break;
             case SORT_KEY_WEIGHT:
                sort_persons(persons, n_persons, &person_cmp_weight);
                break;
            }
            persons_report(fn_rep, persons, n_persons);
        }
    }
    if (persons != NULL) { free_persons(persons, n_persons); }
    return rc;
} /* main */

